# -*- coding: utf-8 -*-
from django import template
from django.db import models, transaction
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django.utils.functional import update_wrapper
from django.contrib.admin.util import unquote, lookup_field, display_for_field, get_deleted_objects
from django.utils.encoding import force_unicode, smart_unicode
from django.shortcuts import render_to_response
from django.http import Http404, HttpResponseRedirect, HttpResponse, HttpResponseForbidden
from django.utils.html import escape, strip_tags
from django.views.decorators.csrf import csrf_protect
from django.conf import settings
from django.utils.decorators import method_decorator
from django.core.exceptions import PermissionDenied, ValidationError
from django.utils.http import urlquote
from django.contrib.admin.util import flatten_fieldsets

from jumbo.core.urlresolvers import reverse
from jumbo.forms.widgets import DateTimeWidget, DateWidget
from jumbo.utils.functions import render_csv
from jumbo.middleware.thread_local import get_request

csrf_protect_m = method_decorator(csrf_protect)

# GET parameter for the URL to return to after change/add views. This lets the
# admin save the state of the changelist page through the change/add page.
RETURN_GET_PARAM = '_return_to'

def set_active(modeladmin, request, queryset):
    rows_updated = 0
    for obj in queryset:
        obj.status = 1
        obj.save()
        rows_updated += 1
    if rows_updated == 1:
        message_bit = ugt("1 object was")
    else:
        message_bit = ugt("%(row)s objects were") % {'row': rows_updated}
    modeladmin.message_user(request, ugt("%(msg)s successfully actived.") % \
                                                  {'msg': message_bit})
set_active.short_description = _("Set %(verbose_name_plural)s as active")
admin.site.add_action(set_active)


def set_disactive(modeladmin, request, queryset):
    rows_updated = 0
    for obj in queryset:
        obj.status = 0
        obj.save()
        rows_updated += 1
    if rows_updated == 1:
        message_bit = ugt("1 object was")
    else:
        message_bit = ugt("%(row)s objects were") % {'row': rows_updated}
    modeladmin.message_user(request, ugt("%(msg)s successfully disactived.") % \
                                                  {'msg': message_bit})
set_disactive.short_description = _("Set %(verbose_name_plural)s as disactive")
admin.site.add_action(set_disactive)

def export_csv(modeladmin, request, queryset):
    from django.http import HttpResponse

    if hasattr(modeladmin, 'list_display_csv'):
        fields_list = modeladmin.list_display_csv
    else:
        fields_list = modeladmin.list_display
    plural_name = ugt(modeladmin.model._meta.verbose_name_plural)
    rows = []

    title = list(fields_list)
    title.append('\n')
    if 'action_checkbox' in title:
        title.remove('action_checkbox')
    title = ";".join(title)
    rows.append(title)

    for obj in queryset:
        row = []
        for field in fields_list:
            if field == 'action_checkbox':
                continue
            f, attr, value = lookup_field(field, obj, modeladmin)
            if f is None:
                result_repr = strip_tags(smart_unicode(value))
            else:
                result_repr = display_for_field(value, f)
            # Tolgo eventuali 'a capo'
            result_repr = result_repr.replace('\r\n', ' ')
            row.append(result_repr)
        row.append('\n')
        row = ";".join(row)
        rows.append(row)

    return render_csv(''.join(rows), '%s.csv' % plural_name)
export_csv.short_description = _("Export selected %(verbose_name_plural)s in csv")
admin.site.add_action(export_csv)


#def export_all_csv(modeladmin, request, queryset):
#    if hasattr(modeladmin, 'list_display_csv'):
#        fields_list = modeladmin.list_display_csv
#    else:
#        fields_list = modeladmin.list_display
#    plural_name = ugt(modeladmin.model._meta.verbose_name_plural)
#    rows = []
#    modello = modeladmin.model
#    for obj in modello.objects.all():
#        row = []
#        for col in fields_list:
#            if col == 'action_checkbox':
#                continue
#            try:
#                row.append(force_unicode(getattr(obj, col)))
#            except:
#                met=getattr(modeladmin, col)
#                row.append(force_unicode(met(obj)))
#        row.append('\n')
#        row = ";".join(row)
#        rows.append(row)
#    return render_csv(''.join(rows), '%s.csv' % plural_name)
#export_all_csv.short_description = _("Export all %(verbose_name_plural)s")
#admin.site.add_action(export_all_csv)


class SettingsAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('general admin'), {'fields': (('name', 'type'), 'value',
                                          'description')}),
    )
    list_display = ('name', 'value', 'type', 'description')
    list_display_links = ('name', )
    list_per_page = 30


class ExtendibleModelAdmin(admin.ModelAdmin):
    class Media:
        js = ['admin/js/collapsed_stacked_inlines.js', 'admin/js/detail_fancybox.js']

    no_display_links = True
    list_per_page = 30
    use_fancybox = False
    father_foreignkey_fancybox  = None

    def get_readonly_fields(self, request, obj=None):
        readonly = ()
        fields = []
        if self.declared_fieldsets:
            fields = flatten_fieldsets(self.declared_fieldsets)

        if 'creator' in fields:
            readonly = readonly + ('creator',)
        if 'date_create' in fields:
            readonly = readonly + ('date_create',)
        if 'last_modifier' in fields:
            readonly = readonly + ('last_modifier',)
        if 'date_last_modify' in fields:
            readonly = readonly + ('date_last_modify',)
        return self.readonly_fields + readonly


    def get_changelist(self, request):
        from jumbo.admin.main import JChangeList
        return JChangeList


    def _getobj(self, request, object_id):
            opts = self.model._meta
            app_label = opts.app_label

            try:
                obj = self.queryset(request).get(pk=unquote(object_id))
            except self.model.DoesNotExist:
                obj = None

            if obj is None:
                raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {'name': force_unicode(opts.verbose_name), 'key': escape(object_id)})
            return obj


    def _wrap(self, view):
        def wrapper(*args, **kwargs):
            return self.admin_site.admin_view(view)(*args, **kwargs)
        return update_wrapper(wrapper, view)


    def _view_name(self, name):
        info = self.model._meta.app_label, self.model._meta.module_name, name
        return '%s_%s_%s' % info


    def has_change_permission(self, request, obj=None):
        has_change_permission = super(ExtendibleModelAdmin, self).has_change_permission(request, obj)
        if obj is None:
            if has_change_permission or \
               request.user.has_perm(self.model._meta.app_label + '.' + 'view_%s' % self.model._meta.module_name):
                return True
            return False
        else:
            if hasattr(obj, "can_be_modified"):
                if obj.can_be_modified():
                    return True
                else:
                    return False
            return has_change_permission


    def has_delete_permission(self, request, obj=None):
        has_delete_permission = super(ExtendibleModelAdmin, self).has_delete_permission(request, obj)
        if hasattr(obj, "can_be_delete"):
            if obj.can_be_delete():
                return True
            else:
                return False

        return has_delete_permission


    def get_value_obj(self, obj, attribute):
        value = None
        try:
            value = getattr(obj, attribute)
        except:
            try:
                if hasattr(self, attribute):
                    value = getattr(self, attribute)(obj)
            except Exception, e:
                pass
        return value


    def detail(self, request, object_id, **kwargs):
        obj = self._getobj(request, object_id)
        opts = self.model._meta
        detail_fields = []
        d_values = {}
        template_name = kwargs.get('template_name', 'detail_form.html')

        if hasattr(self, 'detail_display'):
            for f in self.detail_display:
                # Se e' una lista aggiunga tt i suoi campi ai detail_fields
                if type(f) == type([]) or type(f) == type(()):
                    detail_fields.append(f)
                    for subf in f:
                        d_values[subf] = self.get_value_obj(obj, subf)
                # Se la funzione e' presente nell'admin la chiamo e aggiungo il
                # valore ai detail fields
                elif hasattr(self, f):
                    detail_fields.append([f])
                    d_values[f] = getattr(self, f)(obj)
                # altrimenti cerco il valore dell'oggetto in maniera normale
                else:
                    detail_fields.append([f])
                    d_values[f] = self.get_value_obj(obj, f)

        context = {
            'title': _('Detail %s') % force_unicode(opts.verbose_name),
            'object_id': object_id,
            'obj': obj,
            'is_popup': request.REQUEST.has_key('_popup'),
            'root_path': self.admin_site.root_path,
            'app_label': opts.app_label,
            'opts': opts,
            'detail_fields': detail_fields,
            'd_values': d_values,
            'has_change_permission': self.has_change_permission(request, obj),
        }
        context_instance = template.RequestContext(request, current_app=self.admin_site.name)
        return render_to_response(["admin/%s/%s/%s" % (opts.app_label, opts.object_name.lower(), template_name),
                                   "admin/%s/%s" % (opts.app_label, template_name),
                                   "admin/%s" % template_name], context,
                                   context_instance=context_instance)


    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(ExtendibleModelAdmin, self).get_urls()

        my_urls = patterns('',
            url(r'^(.+)/detail/$',
                self._wrap(self.detail),
                name=self._view_name('detail')),
            )
        return my_urls + urls


    def get_edit_icon(self, result):
        opts = self.model._meta
        can_be_modified = False
        # Se il modello ha la funzione can_be_modified la chiamo per fare
        # i controlli relativi a quel modello
        if hasattr(result, "can_be_modified"):
            if result.can_be_modified():
                can_be_modified = True
        # Se non ce l'ha non faccio controlli e visualizzo l'icona
        else:
            can_be_modified = True

        #return "%s/?%s=%s" % (quote(getattr(result, self.pk_attname)), RETURN_GET_PARAM, urlquote(self.this_clist_url))

        request = get_request()

        if can_be_modified:
            print (reverse('admin:%s_%s_change' % (opts.app_label, opts.object_name.lower()), result.pk), RETURN_GET_PARAM, urlquote(request.get_full_path()))
            return "<a href=%(reverse_url)s><img src='%(url)sadmin/images/edit.gif' alt='%(window_title)s' title='%(window_title)s'/></a>" % {
            'reverse_url': "%s?%s=%s" % (reverse('admin:%s_%s_change' % (opts.app_label, opts.object_name.lower()), result.pk),
                                         RETURN_GET_PARAM, urlquote(request.get_full_path())
                                         ),
            'url':settings.MEDIA_URL,
            'window_title':ugt("Edit %s" % opts.object_name.lower())
        }
        return ""
    get_edit_icon.short_description = _("E")
    get_edit_icon.allow_tags = True


    def get_delete_icon(self, result):
        opts = self.model._meta
        can_be_delete = False
        # Se il modello ha la funzione can_be_modified la chiamo per fare
        # i controlli relativi a quel modello
        if hasattr(result, "can_be_delete"):
            if result.can_be_delete():
                can_be_delete = True
        # Se non ce l'ha non faccio controlli e visualizzo l'icona
        else:
            can_be_delete = True

        if can_be_delete:
            param = ""
            if self.use_fancybox:
                param='?_popup=1&nobuttons=1'
            return "<a href=%(reverse_url)s%(param)s><img src='%(url)sadmin/images/del.gif' alt='%(window_title)s' title='%(window_title)s'/></a>" % {
                'reverse_url':reverse('admin:%s_%s_delete' % (opts.app_label, opts.object_name.lower()), result.pk),
                'param': param,
                'url':settings.MEDIA_URL,
                'window_title':ugt("Delete %s" % opts.object_name.lower())
            }
        return ""
    get_delete_icon.short_description = _("D")
    get_delete_icon.allow_tags = True

    def get_detail_icon(self, result):
        opts = self.model._meta
        return "<a class='iframe' href=%(reverse_url)s?_popup=1&nobuttons=1><img src='%(url)sadmin/images/search.png' alt='%(window_title)s' title='%(window_title)s'/></a>" % {
            'reverse_url':reverse('%s:%s_%s_detail' % (self.admin_site.name, opts.app_label, opts.object_name.lower()), result.pk),
            'url':settings.MEDIA_URL,
            'window_title':ugt("Detail %s" % opts.object_name.lower())
        }
    get_detail_icon.allow_tags = True
    get_detail_icon.short_description = _("V")

    def get_status_icon(self, obj):
        return obj.status
    get_status_icon.short_description = _("ST")
    get_status_icon.boolean = True
    get_status_icon.admin_order_field = 'status'

    def get_response_fancybox(self, obj, is_change=False):
        if self.father_foreignkey_fancybox:
            original_father = getattr(obj, self.father_foreignkey_fancybox)
            original_id = original_father.id
            app_name = original_father._meta.app_label
            model_name = original_father._meta.object_name

            return HttpResponse("""
                <script type="text/javascript">
                    parent.$.fancybox.close();
                    parent.Dajaxice.jumbo.apps.dajax.common_fancybox_inline(parent.Dajax.process,
                        {'object_id':%(original)s,
                        'app':'%(app)s',
                        'model':'%(model)s',
                        'is_change': '%(is_change)s'})

                </script>""" % {'media_url': settings.MEDIA_URL,
                                'model'    : model_name,
                                'app'      : app_name,
                                'original' : original_id,
                                'is_change': is_change})
        else:
            return HttpResponse("""
                <script type="text/javascript">
                    parent.$.fancybox.close();
                    parent.window.location.reload();
                </script>""")


    def response_add(self, request, obj, post_url='../', post_url_continue='../%s/', msg=None):
        """
        Determines the HttpResponse for the add_view stage.
        Aggiunta possibilità di personalizzare il post_url e il messaggio di conferma
        """
        opts = obj._meta
        pk_value = obj._get_pk_val()

        if not msg:
            msg = ugt('The %(name)s "%(obj)s" was added successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj)}

        # Here, we distinguish between different save types by checking for
        # the presence of keys in request.POST.
        if request.POST.has_key("_continue"):
            self.message_user(request, msg + ' ' + ugt("You may edit it again below."))
            if request.POST.has_key("_popup"):
                post_url_continue += "?_popup=1"
            return HttpResponseRedirect(post_url_continue % pk_value)

        if request.POST.has_key("_popup"):
            if self.use_fancybox:
                return self.get_response_fancybox(obj)

            return HttpResponse('<script type="text/javascript">opener.dismissAddAnotherPopup(window, "%s", "%s");</script>' % \
                # escape() calls force_unicode.
                (escape(pk_value), escape(obj)))
        elif request.POST.has_key("_addanother"):
            self.message_user(request, msg + ' ' + (ugt("You may add another %s below.") % force_unicode(opts.verbose_name)))
            return HttpResponseRedirect(request.path)
        else:
            self.message_user(request, msg)

            # Figure out where to redirect. If the user has change permission,
            # redirect to the change-list page for this object. Otherwise,
            # redirect to the admin index.
            if RETURN_GET_PARAM in request.GET:
                post_url = request.GET[RETURN_GET_PARAM]
            elif self.has_change_permission(request, None):
                post_url = post_url
            else:
                post_url = '../../../'
            return HttpResponseRedirect(post_url)


    def response_change(self, request, obj, post_url='../', msg=None):
        """
        Determines the HttpResponse for the change_view stage.
        Aggiunta possibilità di personalizzare il post_url e il messaggio di conferma
        """
        opts = obj._meta
        pk_value = obj._get_pk_val()

        if not msg:
            msg = ugt('The %(name)s "%(obj)s" was changed successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj)}

        if request.POST.has_key("_popup") and self.use_fancybox:
            return self.get_response_fancybox(obj, is_change=True)

        if request.POST.has_key("_continue"):
            self.message_user(request, msg + ' ' + ugt("You may edit it again below."))
            if request.REQUEST.has_key('_popup'):
                return HttpResponseRedirect(request.path + "?_popup=1")
            else:
                return HttpResponseRedirect(request.path)
        elif request.POST.has_key("_saveasnew"):
            msg = ugt('The %(name)s "%(obj)s" was added successfully. You may edit it again below.') % {'name': force_unicode(opts.verbose_name), 'obj': obj}
            self.message_user(request, msg)
            return HttpResponseRedirect("../%s/" % pk_value)
        elif request.POST.has_key("_addanother"):
            self.message_user(request, msg + ' ' + (ugt("You may add another %s below.") % force_unicode(opts.verbose_name)))
            return HttpResponseRedirect("../add/")
        else:
            self.message_user(request, msg)
            post_url = request.GET.get(RETURN_GET_PARAM, post_url)
            return HttpResponseRedirect(post_url)

    @csrf_protect_m
    def delete_view(self, request, object_id, extra_context=None, js_function=''):
        """ The 'delete' admin view for this model.
            Aggiunta la funzionalità di usare la vista di delete
            anche con fancybox
        """
        opts = self.model._meta
        app_label = opts.app_label

        obj = self.get_object(request, unquote(object_id))

        if not self.has_delete_permission(request, obj):
            context_instance = template.RequestContext(request, current_app=self.admin_site.name)
            return render_to_response("admin/forbidden.html", context_instance=context_instance)

        if obj is None:
            raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {'name': force_unicode(opts.verbose_name), 'key': escape(object_id)})

        # Populate deleted_objects, a data structure of all related objects that
        # will also be deleted.
        (deleted_objects, perms_needed) = get_deleted_objects((obj,), opts, request.user, self.admin_site)

        if request.POST: # The user has already confirmed the deletion.
            if perms_needed:
                raise PermissionDenied

            obj_display = force_unicode(obj)
            self.log_deletion(request, obj, obj_display)
            obj.delete()
            msg = ugt('The %(name)s "%(obj)s" was deleted successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj)}
            #non  viene incluso nel successivo if perche django non passa il parametro _popup al template
            if self.use_fancybox:
                return self.get_response_fancybox(obj, is_change=True)
            else:
                self.message_user(request, msg)
            #12/11/2012 commentiamo questo if perchè secondo noi non viene mai usata
            # in caso di malfunzionamenti riabilitare - Marco e dario
#            if request.POST.has_key("_popup"):
#                if js_function:
#                    js_function = "parent.window.%s();" % js_function
#                return HttpResponse("""
#                <script type="text/javascript" src="/static/admin/js/jquery.min.js"></script>
#                <script type="text/javascript" src="/static/admin/js/fancybox/jquery.fancybox-1.3.4.pack.js"></script>
#                <script type="text/javascript">parent.$.fancybox.close();%s</script>""" % js_function)

            if not self.has_change_permission(request, None):
                return HttpResponseRedirect("../../../../")
            return HttpResponseRedirect("../../")

        context = {
            "title": _("Are you sure?"),
            "object_name": force_unicode(opts.verbose_name),
            "object": obj,
            "deleted_objects": deleted_objects,
            "perms_lacking": perms_needed,
            "opts": opts,
            "root_path": self.admin_site.root_path,
            "app_label": app_label,
        }
        context.update(extra_context or {})
        context_instance = template.RequestContext(request, current_app=self.admin_site.name)
        return render_to_response(self.delete_confirmation_template or [
            "admin/%s/%s/delete_confirmation.html" % (app_label, opts.object_name.lower()),
            "admin/%s/delete_confirmation.html" % app_label,
            "admin/delete_confirmation.html"
        ], context, context_instance=context_instance)


class OrderedModelAdmin(admin.ModelAdmin):
    def _view_name(self, name):
        info = self.model._meta.app_label, self.model._meta.module_name, name
        return '%s_%s_%s' % info


    def _wrap(self, view):
        def wrapper(*args, **kwargs):
            return self.admin_site.admin_view(view)(*args, **kwargs)
        return update_wrapper(wrapper, view)


    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(OrderedModelAdmin, self).get_urls()

        my_urls = patterns('',
            url(r'^(.+)/up/$',
                self._wrap(self.up),
                name=self._view_name('up')),
            url(r'^(.+)/down/$',
                self._wrap(self.down),
                name=self._view_name('down')),
            )
        return my_urls + urls


    def up(self, request, id):
        node = self.model._default_manager.get(pk=id)
        node.up()
        try:
            redirect_to = request.META['HTTP_REFERER']
        except:
            redirect_to = '../../'
        return HttpResponseRedirect(redirect_to)


    def down(self, request, id):
        node = self.model._default_manager.get(pk=id)
        node.down()
        try:
            redirect_to = request.META['HTTP_REFERER']
        except:
            redirect_to = '../../'
        return HttpResponseRedirect(redirect_to)

    def move_actions(self, node):
        info = self.admin_site.name, self.model._meta.app_label, self.model._meta.module_name
        data = []
        if not node.is_first(): # up node
            data.append(u'<a href="%s" class="nodes-up">%s</a>' % (reverse('%s:%s_%s_up' % info, node.id), _('up')))
        if not node.is_last() and not node.is_first():
            data.append(u'<span style="font-weight:normal"> | </span>')
        if not node.is_last(): # down node
            data.append(u'<a href="%s" class="nodes-down">%s</a>' % (reverse('%s:%s_%s_down' % info, node.id), _('down')))
        return u''.join(data)
    move_actions.short_description = _('move')
    move_actions.allow_tags = True
