import calendar
import locale
from datetime import datetime, date, time, timedelta
from django.utils.translation import ugettext, ugettext_lazy as _


class CalendarDay(object):
    """
    Represents a single calendar day
    """
    def __init__(self, day=date.today()):
        self.events=[]     # Lista di eventi della giornata
        self.day = day     # Giorno del calendario
        self.first_datetime_range = datetime(self.day.year, self.day.month, self.day.day, 0, 0, 0)
        self.last_datetime_range = datetime(self.day.year, self.day.month, self.day.day, 23, 59, 59)
        self.current_day = datetime(datetime.today().year,
                                    datetime.today().month,
                                    datetime.today().day) # Giornata odierna

    def get_next_day(self):
        """
        Return the next calendar day
        """
        return CalendarDay(self.day + timedelta(days=1))
    next = get_next_day

    def get_prev_day(self):
        """
        Return the previous calendar day
        """
        return CalendarDay(self.day - timedelta(days=1))
    prev = get_prev_day

    def get_current_year(self):
        """
        Return the calendar day's year
        """
        return CalendarYear(self.day)
    current_year = get_current_year

    def get_current_month(self):
        """
        Return the calendar day's month
        """
        return CalendarMonth(self.day)
    current_month = get_current_month

    def get_current_week(self):
        """
        Return the calendar day's week
        """
        return CalendarWeek(self.day)
    current_week = get_current_week

    def get_first_range_day(self):
        """
        Get the event's lookup range calendar (day and hour)
        """
        first_date = self.day
        first_date_time = datetime(self.day.year, self.day.month, self.day.day, 0, 0, 0)
        return first_date, first_date_time

    def get_last_range_day(self):
        """
        Get the event's lookup range calendar (day and hour)
        """
        last_date = self.day
        last_date_time = datetime(self.day.year, self.day.month, self.day.day, 23, 59, 59)
        return last_date, last_date_time

    def is_busy(self):
        """
        Return True if there are events in the day
        """
        if self.events:
            return True
        else:
            return False

    def is_today(self):
        """
        Return True if the day is Today
        """
        if self.day == self.current_day:
            return True
        return False

    def is_holiday(self):
        """
        Return True if the day is an holiday day
        """
        if self.day.weekday() in (5,6):
            return True
        return False

    def is_monday(self):
        """
        Return True if the day is Monday
        """
        if self.day.weekday() == 0:
            return True
        return False
    
    def is_sunday(self):
        """
        Return True if the day is Sunday
        """
        if self.day.weekday() == 6:
            return True
        return False
    
    def is_hide(self, period):
        """
        Return True if the day is inside the period
        """
        if self.day < period.first_datetime_range.date() or self.day > period.last_datetime_range.date():
            return True
        return False

    def add_event(self, event, field_date_begin=None, field_date_end=None):
        """
        Add an event to the Day

        Keyword arguments:
            event -- The Event Instance to add to the day
        """
        self.events.append(event)

    def add_events(self, list_events):
        """
        Add a list of events to the Day

        Keyword arguments:
            list_events -- A list of Event Instances to add to the day
        """
        self.events += list_events

    def remove_all_events(self):
        """
        Remove all Events Instance for this Day
        """
        self.events = []

    def __unicode__(self):
        return self.day.strftime('%d/%m/%Y')

class CalendarWeek(object):
    """
    Represents a single calendar week
    """
    def __init__(self, day=date.today()):
        self.events=[]     # Lista di eventi della giornata
        self.hide = False
        self.day = day
        self.first_day = self.get_first_day()
        self.last_day = self.get_last_day()
        self.first_datetime_range = datetime(self.first_day.year, self.first_day.month, self.first_day.day, 0, 0, 0)
        self.last_datetime_range = datetime(self.last_day.year, self.last_day.month, self.last_day.day, 23, 59, 59)
        self.days = self.get_days()
        self.day_before = self.first_day - timedelta(days=1)
        self.day_after = self.last_day + timedelta(days=1)
        self.num_week = self.day.isocalendar()[1]
        self.dic_days = {}
        self.days = self.get_days()


    def get_next_week(self):
        """
        Return the next calendar week
        """
        return CalendarWeek(self.last_day + timedelta(days=1))
    next = get_next_week

    def get_prev_week(self):
        """
        Return the prev calendar week
        """
        return CalendarWeek(self.first_day - timedelta(days=7))
    prev = get_prev_week

    def get_current_month(self):
        return CalendarMonth(self.day)

    def get_current_year(self):
        return CalendarYear(self.day)

    def get_first_day(self):
        """
        Return the first day of week
        """
        return self.day - timedelta(days=self.day.weekday())

    def get_last_day(self):
        """
        Return the last day of week
        """
        return self.day - timedelta(days=self.day.weekday()-6)

    def get_days(self):
        """
        Return a list of day for the week's range
        """
        period = []
        self.dic_days = {}
        step_day = self.first_day
        while step_day <= self.last_day:
            day = CalendarDay(step_day)
            period.append(day)
            self.dic_days[step_day] = day
            step_day += timedelta(days=1)
        return period

    def add_event(self, event, field_date_begin, field_date_end=None):
        """
        Add an event to a Day

        Keyword arguments:
            event -- The Event Instance to add to the day
        """
        date_begin = getattr(event, field_date_begin)

        date_end = None
        if field_date_end:
            date_end = getattr(event, field_date_end)

        if not date_end:
            date_end =  date_begin

        mydate = date_begin

        while mydate <= date_end:
            if self.first_datetime_range.year != mydate.year or \
               self.first_datetime_range.month != mydate.month or\
               self.num_week != mydate.isocalendar()[1]:
                mydate = mydate + timedelta(days = 1)
                continue
            dayevent = date(mydate.year, mydate.month, mydate.day)
            self.dic_days[dayevent].add_event(event)
            mydate = mydate + timedelta(days = 1)

    def add_day_event(self, event, dayevent):
        dayevent = date(dayevent.year, dayevent.month, dayevent.day)
        self.dic_days[dayevent].add_event(event)

    def is_busy(self):
        """
        Return True if there are events in the day
        """
        if self.events:
            return True
        else:
            return False

    def __unicode__(self):
        return self.day.strftime('%w, %Y')

class CalendarMonth(object):
    """
    Represents a single calendar month
    """
    def __init__(self, day=date.today()):
        self.events=[]     # Lista di eventi della giornata
        self.hide = False
        self.day = day
        self.first_day = self.get_first_day()
        self.last_day = self.get_last_day()
        self.first_datetime_range = datetime(self.first_day.year, self.first_day.month, self.first_day.day, 0, 0, 0)
        self.last_datetime_range = datetime(self.last_day.year, self.last_day.month, self.last_day.day, 23, 59, 59)
        self.day_before = self.first_day - timedelta(days=1)
        self.day_after = self.last_day + timedelta(days=1)
        self.prev_monday = self.first_day - timedelta(days=self.first_day.weekday())
        self.next_sunday = self.last_day - timedelta(days=self.last_day.weekday()-6)
        self.dic_weeks = {}
        self.weeks = self.get_weeks()

    def get_next_month(self):
        """
        Return the next calendar month
        """
        return CalendarMonth(self.last_day+timedelta(days=1))
    next = get_next_month

    def get_prev_month(self):
        """
        Return the prev calendar month
        """
        return CalendarMonth(self.first_day-timedelta(days=1))
    prev = get_prev_month

    def get_current_year(self):
        """
        Return the current month's year
        """
        return CalendarYear(self.day)

    def get_prev_year(self):
        """
        Return the previous year
        """
        first_day_py = date(self.first_day.year-1, 1, 1)
        return Year(first_day_py)

    def get_next_year(self):
        """
        Return the next year
        """
        last_day_ny = date(self.last_day.year+1, 12, 31)
        return Year(last_day_ny)

    def get_first_day(self):
        """
        Return the first day of week
        """
        return date(self.day.year, self.day.month, 1)

    def get_last_day(self):
        """
        Return the last day of week
        """
        last_of_month = calendar.monthrange(self.day.year, self.day.month)[1]
        return date(self.day.year, self.day.month, last_of_month)

    def get_weeks(self):
        """
        Return a list of week for the month's range
        """
        weeks = []
        
        step_day = self.prev_monday
        while step_day <= self.next_sunday:
            cw = CalendarWeek(step_day)
            weeks.append(cw)
            w = step_day.isocalendar()[1]
            self.dic_weeks[w] = cw
            step_day += timedelta(days=7)
        return weeks

    def add_event(self, event, field_date_begin, field_date_end=None):
        """
        Add an event to a Day

        Keyword arguments:
            event -- The Event Instance to add to the day
        """
        date_begin = getattr(event, field_date_begin)

        date_end = None
        if field_date_end:
            date_end = getattr(event, field_date_end)

        if not date_end:
            date_end =  date_begin

        date = date_begin
        while date <= date_end:
            if self.first_datetime_range.year != date.year or self.first_datetime_range.month != date.month:
                date = date + timedelta(days = 1)
                continue
            self.dic_weeks[date.isocalendar()[1]].add_day_event(event, date)
            date = date + timedelta(days = 1)


    def add_day_event(self, event, dayevent):
        self.dic_weeks[dayevent.isocalendar()[1]].add_day_event(event, dayevent)

    def is_busy(self):
        """
        Return True if there are events in the day
        """
        if self.events:
            return True
        else:
            return False

    def string_month(self):
        locale.setlocale(locale.LC_ALL, '')
        return self.first_day.strftime('%B')

    def __unicode__(self):
        return self.first_day.strftime('%m/%Y')

class CalendarYear(object):
    """
    Represents a single calendar year
    """
    def __init__(self, day=date.today()):
        self.hide = False
        self.day = day
        self.first_day = self.get_first_day()
        self.last_day = self.get_last_day()
        self.first_datetime_range = datetime(self.first_day.year, self.first_day.month, self.first_day.day, 0, 0, 0)
        self.last_datetime_range = datetime(self.last_day.year, self.last_day.month, self.last_day.day, 23, 59, 59)
        self.day_before = self.first_day - timedelta(days=1)
        self.day_after = self.last_day + timedelta(days=1)
        self.dic_months = {}
        self.months = self.get_months()
        
    def get_next_year(self):
        """
        Return the next calendar year
        """
        return CalendarYear(self.last_day+timedelta(days=1))
    next = get_next_year

    def get_prev_year(self):
        """
        Return the previous calendar year
        """
        return CalendarYear(self.first_day-timedelta(days=1))
    prev = get_prev_year

    def get_first_day(self):
        """
        Return the first day of year
        """
        return date(self.day.year, 1, 1)

    def get_last_day(self):
        """
        Return the last day of year
        """
        return date(self.day.year, 12, 31)

    def get_months(self):
        """
        Return a list of month for the year's range
        """
        months = []
        for m in range(12):
            m += 1
            step_day = date(self.first_day.year, m, 1)
            cm = CalendarMonth(step_day)
            months.append(cm)
            self.dic_months[m] = cm
        return months

    def add_event(self, event, field_date_begin, field_date_end=None):
        """
        Add an event to a Day

        Keyword arguments:
            event -- The Event Instance to add to the day
        """
        date_begin = getattr(event, field_date_begin)

        date_end = None
        if field_date_end:
            date_end = getattr(event, field_date_end)

        if not date_end:
            date_end =  date_begin

        date = date_begin

        while date <= date_end:
            if self.first_datetime_range.year != date.year:
                date = date + timedelta(days = 1)
                continue
            self.dic_months[date.month].add_day_event(event, date)
            date = date + timedelta(days = 1)

    def is_busy(self):
        """
        Return True if there are events in the day
        """
        if self.events:
            return True
        else:
            return False

    def __unicode__(self):
        return self.first_day.strftime('%Y')

class TimeGrid(object):
    """
    Return the range of a day
    """
    def get_range(self, events=None):
        min_value = 8
        max_value = 19
        max_list = []
        min_list = []
        if events:
            for event in events:
                if int(event.time_begin.strftime("%H"))<min_value:
                    min_value = int(event.time_begin.strftime("%H"))
                if int(event.time_end.strftime("%H"))>max_value:
                    max_value = int(event.time_end.strftime("%H"))
        return range(min_value, max_value+1)


class CalendarSheet(object):
    """
    Represents the timesheet of a day
    """
 
    def __init__(self, c):
        self.real_events = [] #EVENTI REALI CHE VERRANNO VISUALIZZATI NEL CALENDARIO ALLA SPECIFICA ORA
        self.fake_events = [] #EVENTI GIA' RAPPRESENTATI CHE NON VERRANNO VISUALIZZATI
        self.hour = c
    
    def count_events(self):
        return len(self.events)
    
    def add_real_event(self, event):
        self.real_events.append(event)
        return len(self.real_events)-1        
    
    def add_fake_event(self, event, position=None):
        # SE LA POSIZIONE PASSATA E' NONE:      
        if position==None:
            for i, e in enumerate(self.fake_events):
                if self.fake_events[i]==None:
        # SE LA POSIZIONE ESISTE, ASSEGNO L'EVENTO NELLA POSIZIONE NONE
                    self.fake_events[i] = event
                    return i
        # ALTRIMENTI LO APPENDO ALLA FINE
            self.fake_events.append(event)
            return len(self.fake_events) - 1
        else:
        # SE LA POSIZIONE E' UN INTERO:
            for i in range(position+1):
                try: 
        #VERIFICO CHE LA LISTA ABBIA UN INDICE SU QUELL'INTERO
                    self.fake_events[i]
                    if i == position:
        #E ASSEGNO L'APPUNTAMENTO
                        self.fake_events[i] = event
                except:
        # ALTRIMENTI APPENDO:
                    if i == position:
                # L'EVENTO SE LA POSIZIONE CICLATA E' UGUALE ALLA POSIZIONE PASSATA
                        self.fake_events.append(event)
                # APPENDO NONE
                    else:
                        self.fake_events.append(None)

        
    def __unicode__(self):
        return self.hour
    
