# -*- coding: utf-8 -*-
# Country and AdministrativeArea models and data are taken from satchmo
# (http://www.satchmoproject.com)

from django.db import models
from django.utils.translation import ugettext_lazy as _

CONTINENTS = (
    ('AF', _('Africa')),
    ('NA', _('North America')),
    ('EU',  _('Europe')),
    ('AS', _('Asia')),
    ('OC',  _('Oceania')),
    ('SA', _('South America')),
    ('AN', _('Antarctica'))
)

AREAS = (
    ('a', _('Another')),
    ('i', _('Island')),
    ('ar', _('Arrondissement')),
    ('at', _('Atoll')),
    ('ai', _('Autonomous island')),
    ('ca', _('Canton')),
    ('cm', _('Commune')),
    ('co', _('County')),
    ('dp', _('Department')),
    ('de', _('Dependency')),
    ('dt', _('District')),
    ('dv', _('Division')),
    ('em', _('Emirate')),
    ('gv', _('Governorate')),
    ('ic', _('Island council')),
    ('ig', _('Island group')),
    ('ir', _('Island region')),
    ('kd', _('Kingdom')),
    ('mu', _('Municipality')),
    ('pa', _('Parish')),
    ('pf', _('Prefecture')),
    ('pr', _('Province')),
    ('rg', _('Region')),
    ('rp', _('Republic')),
    ('sh', _('Sheading')),
    ('st', _('State')),
    ('sd', _('Subdivision')),
    ('sj', _('Subject')),
    ('ty', _('Territory')),
)

class Country(models.Model):
    """
    International Organization for Standardization (ISO) 3166-1 Country list
    """
    name = models.CharField(
        max_length=128,
        db_index=True,
        verbose_name=_('Official name (CAPS)'),
    )
    printable_name = models.CharField(
        max_length=128,
        db_index=True,
        verbose_name=_('Country name'),
    )
    iso2_code = models.CharField(
        max_length=2,
        unique=True,
        db_index=True,
        verbose_name=_('ISO alpha-2'),
    )
    iso3_code = models.CharField(
        max_length=3,
        unique=True,
        verbose_name=_('ISO alpha-3'),
    )
    numcode = models.PositiveSmallIntegerField(
        null=True,
        blank=True,
        verbose_name=_('ISO numeric'),
    )
    active = models.BooleanField(
        default=True,
        verbose_name=_('Country is active'),
    )
    continent = models.CharField(
        choices=CONTINENTS,
        max_length=2,
        verbose_name=_('Continent'),
    )
    admin_area = models.CharField(
        choices=AREAS,
        max_length=2,
        blank=True,
        null=True,
        db_index=True,
        verbose_name=_('Administrative Area'),
    )

    class Meta:
        verbose_name = _('Country')
        verbose_name_plural = _('Countries')
        ordering = ('name',)

    def __unicode__(self):
        return self.printable_name


class AdministrativeArea(models.Model):
    """
    Administrative Area level 1 for a country.  For the US, this would be the states
    """
    country = models.ForeignKey(
        Country,
        related_name='administrative_areas',
    )
    name = models.CharField(
        max_length=60,
        db_index=True,
        verbose_name=_('Admin Area name'),
    )
    abbrev = models.CharField(
        max_length=3,
        blank=True,
        null=True,
        verbose_name=_('Postal Abbreviation'),
    )
    active = models.BooleanField(
        default=True,
        verbose_name=_('Area is active'),
    )

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('Administrative Area')
        verbose_name_plural = _('Administrative Areas')
        ordering = ('name',)

