import datetime
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django.conf import settings
from django import forms

from jumbo.admin.options import OrderedModelAdmin, ExtendibleModelAdmin
from jumbo.core.urlresolvers import reverse
from jumbo.apps.project.models import Project, ProjectNumber

class ProjectNumberAdmin(ExtendibleModelAdmin):
    fieldsets = (
        (_('logs admin'), {
            'fields'  : ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None           , {
            'fields'  : (
                ('year', 'last_date'),
                ('last_onumber', 'last_cnumber',),
            )
        }),
    )
    list_display = (
        'year', 'last_date', 'last_onumber', 'last_cnumber',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )

class ProjectSearchForm(forms.ModelForm):
    class Meta:
        model          = Project
        fields         = ('status', 'organization', 'name')
        display_fields = (
            'status', 'organization', 'name',
            'procedures__number',
            ('date_start__year', 'search_in_all_years'),
        )

    def __init__(self, *args, **kws):
        super(ProjectSearchForm, self).__init__(*args, **kws)
        self.fields['procedures__number'] = forms.CharField(label=_("Procedure number"), required=False)
        self.fields['search_in_all_years'] = forms.BooleanField(label=_("Search In All Years"), required=False)
        self.fields['date_start__year'] = forms.CharField(label=_("Year"), required=False)

    def change_search_filter(self, lookup_params, qs):
        if (
            not lookup_params.has_key('date_start__year') and
            not lookup_params.has_key('search_in_all_years')
            ):
            qs = qs.filter(date_start__year = datetime.date.today().year)

        if lookup_params.has_key('search_in_all_years'):
            del lookup_params['search_in_all_years']
        return lookup_params, qs

class ProjectAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("project/css/admin_project.css",)}

    search_form        = ProjectSearchForm
    fieldsets          = (
        (_('logs admin'), {
            'fields'  : ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None           , {
            'fields'  : (
                'status',
                'organization',
                'name',
                'description',
                ('date_start', 'date_end',),
            )
        }),
    )
    search_fields      = ['name', 'organization__code', 'organization__name']
    list_display       = [
        'get_status_icon', 'get_organization', 'name', 'get_date_start',
        'get_date_end', 'get_detail_icon', 'get_edit_icon', 'get_ticket_icon',
        'get_document_icon'
    ]

    if not "ticket" in settings.INSTALLED_APPS:
        list_display.remove('get_ticket_icon')
    if not "document" in settings.INSTALLED_APPS:
        list_display.remove('get_document_icon')

    list_display_csv   = ('status', 'organization', 'name', 'get_date_start', 'get_date_end')

    def get_ticket_icon(self, obj):
        try:
            tp = obj.ticketproject.id
        except:
            tp = None
        ## se non esiste aggiungo
        if not tp:
            link = """
                <a class='iframe' href="%(href)s?_popup=1&project_ptr_id=%(obj_id)s" title="%(title)s">
                    <img src='%(url)sadmin/images/add_yellow.png' alt='%(title)s' title='%(title)s'/>
                </a>
            """ % {
                'href'      :   reverse('admin:ticket_ticketproject_add'),
                'obj_id'    :   obj.id,
                'title'     :   ugt("Add Ticket Project"),
                'url'       :   settings.MEDIA_URL,
            }
        ## se esiste modifico
        else:
            link = """
                <a class='iframe' href="%(href)s?_popup=1" title="%(title)s">
                    <img src='%(url)sadmin/images/edit.gif' alt='%(title)s' title='%(title)s'/>
                </a>
            """ % {
                'href'      :   reverse('admin:ticket_ticketproject_change', tp),
                'title'     :   ugt("Edit Ticket Project"),
                'url'       :   settings.MEDIA_URL,
            }
        return link
    get_ticket_icon.short_description = _("Tic")
    get_ticket_icon.allow_tags = True

    def get_document_icon(self, obj):
        try:
            dp = obj.documentproject.id
        except:
            dp = None
        ## se non esiste aggiungo
        if not dp:
            link = """
                <a class='iframe' href="%(href)s?_popup=1&project_ptr_id=%(obj_id)s" title="%(title)s">
                    <img src='%(url)sadmin/images/add_yellow.png' alt='%(title)s' title='%(title)s'/>
                </a>
            """ % {
                'href'      :   reverse('admin:document_documentproject_add'),
                'obj_id'    :   obj.id,
                'title'     :   ugt("Add Document Project"),
                'url'       :   settings.MEDIA_URL,
            }
        ## se esiste
        else:
            link = """
                <a class='iframe' href="%(href)s?_popup=1" title="%(title)s">
                    <img src='%(url)sadmin/images/edit.gif' alt='%(title)s' title='%(title)s'/>
                </a>
            """ % {
                'href'      :   reverse('admin:document_documentproject_change', dp),
                'title'     :   ugt("Edit Document Project"),
                'url'       :   settings.MEDIA_URL,
            }
        return link
    get_document_icon.short_description = _("Doc")
    get_document_icon.allow_tags = True

    def get_date_start(self, obj):
        return obj.date_start.strftime("%d/%m/%Y")
    get_date_start.short_description = _('Date Start')
    get_date_start.admin_order_field = 'date_start'

    def get_date_end(self, obj):
        return obj.date_end.strftime("%d/%m/%Y")
    get_date_end.short_description = _('Date End')
    get_date_end.admin_order_field = 'date_end'

    def get_organization(self, obj):
        detail = """<a href="%(href)s" title="%(title)s">%(org)s</a>""" % {
            'href'      :   reverse('admin:organization_organization_detail', obj.organization.id),
            'title'     :   ugt("View Organization Detail"),
            'org'       :   obj.organization.name,
        }
        return detail
    get_organization.short_description = _("Organization")
    get_organization.allow_tags = True
    get_organization.admin_order_field = 'organization__name'

admin.site.register(Project, ProjectAdmin)
admin.site.register(ProjectNumber, ProjectNumberAdmin)
