# -*- coding: utf-8 -*-

from datetime import datetime, date

from django.db import models
from django.contrib.auth.models import User, Group, Permission
from django.utils.translation import ugettext_lazy as _
from django.conf import settings as ls

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from organization.models import Organization #, Contact

############################
# NUMERO PROGETTO          #
############################
class ProjectNumber(DateModel, UserModel):
    year = models.IntegerField (
        unique = True,
        db_index = True,
        verbose_name = _("year"),
    )
    last_date = models.DateField(
        db_index = True,
        verbose_name = _('last date')
    )
    last_onumber = models.PositiveIntegerField(
        db_index = True,
        blank = True, null = True,
        verbose_name = _("last number")
    )
    last_cnumber = models.PositiveIntegerField(
        db_index = True,
        blank = True, null = True,
        verbose_name = _("last number")
    )
    class Meta:
        verbose_name = _("project number")
        verbose_name_plural = _("project numbers")
        permissions = (
            ("view_projectnumber"    , "Can view project number"),
        )

############################
# PROGETTO                 #
############################
PSTATUS_CODES = (
    (1, 'Open'),
    (0, 'Closed'),
)

class Project(DateModel, UserModel):
    status = models.IntegerField (
        default = '1',
        choices = PSTATUS_CODES,
        db_index = True,
        verbose_name = _("Status"),
    )
    ## ForeignKey
    organization = models.ForeignKey (
        Organization,
        null = True, blank = True,
        related_name = "projects",
        db_index = True,
        verbose_name = _("Organization")
    )
    ## Campi
    o_number = models.CharField (
        max_length = 50,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Initial Number"),
        db_column = "pnumber"
    )
    c_number = models.CharField (
        max_length = 50,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("confirm Number"),
        db_column = "cnumber"
    )
    o_date = models.DateField (
        db_index = True,
        blank = True, null = True,
        verbose_name = _("Initial Date"),
    )
    c_date = models.DateField (
        db_index = True,
        blank = True, null = True,
        verbose_name = _("Confirm Date"),
    )
    name = models.CharField (
        max_length = 200,
        db_index = True,
        verbose_name = _("Project"),
    )
    description = models.TextField (
        null = True, blank = True,
        verbose_name = _("Description"),
    )
    date_start = models.DateField (
        db_index = True,
        default = date.today,
        verbose_name = _("Start Date"),
    )
    date_end = models.DateField (
        null = True, blank = True,
        db_index = True,
        verbose_name =  _("End Date"),
    )
    # inizio lavori - cantiere
    date_start_site = models.DateField (
        db_index = True,
        null = True, blank = True,
        verbose_name = _("Start Site Date"),
    )
    accepted = models.BooleanField (
        ## PROGETTO accettato - da offerta/opportunita' diventa commessa/progetto
        default = False,
        db_index = True,
        verbose_name = _("Accepted"),
    )

    def save(self, force_insert=False, force_update=False):
        from jumbo.apps.project.utils import create_update_number
        if not self.accepted and not self.o_number:
            ## se il progetto non è stato ancora accettato e non ha un numero
            self.o_number = create_update_number(tproject="o")
            self.o_date = date.today()
        if self.accepted and not self.c_number:
            ## se il progetto è stato accettato e non ha un numero
            self.c_number = create_update_number(tproject="c")
            self.c_date = date.today()
        super(Project, self).save(force_insert=force_insert, force_update=force_update)

    class Meta:
        verbose_name = _("Project")
        verbose_name_plural = _("Projects")
        permissions = (
            ("can_view_projects"   , "Can view projects"),
            ("list_all_projects"   , "Can list all projects"),
        )
        ordering = ['name']

    def __unicode__(self):
        description = self.get_description()
        return unicode(description)

    def get_title(self):
        o_date = ''
        if self.o_date:
            o_date = " del %s" % self.o_date.strftime('%d/%m/%Y')
        description = "%s - OFFERTA %s%s" % (
            self.name,
            self.o_number,
            o_date
        )
        if "quotation" in ls.INSTALLED_APPS:
            if self.last_cus_quotation:
                description = "%s - OFFERTA %s del %s" % (
                    self.last_cus_quotation.description,
                    self.last_cus_quotation.number,
                    self.last_cus_quotation.date_q.strftime('%d/%m/%Y')
                )
                if self.last_cus_quotation.accepted and self.last_cus_procedure:
                    description = "%s - COMMESSA %s del %s" % (
                        self.last_cus_procedure.description,
                        self.last_cus_procedure.number,
                        self.last_cus_procedure.date_p.strftime('%d/%m/%Y')
                    )
        return description
    get_title.short_description = _("Title")

    if not getattr(ls, 'PROJECT_SHORT_DESCRIPTION', False):
        def get_description(self):
            description = ''
            if self.description:
                description = self.description
            else:
                description = self.name
    
            o_date = ''
            if self.o_date:
                o_date = " del %s" % self.o_date.strftime('%d/%m/%Y')
            description = "%s - OFFERTA %s%s" % (
                description,
                self.o_number,
                o_date
            )
            if "quotation" in ls.INSTALLED_APPS:
                if self.last_cus_quotation:
                    description = "%s - OFFERTA %s del %s" % (
                        self.last_cus_quotation.description,
                        self.last_cus_quotation.number,
                        self.last_cus_quotation.date_q.strftime('%d/%m/%Y')
                    )
                    if self.last_cus_quotation.accepted and self.last_cus_procedure:
                        description = "%s - COMMESSA %s del %s" % (
                            self.last_cus_procedure.description,
                            self.last_cus_procedure.number,
                            self.last_cus_procedure.date_p.strftime('%d/%m/%Y')
                        )
            return description
        get_description.short_description = _("Offerta/Commessa")
    # per TETTI e AFFINI
    else:
        def get_description(self):
            o_date = ''
            if self.o_date:
                o_date = " del %s" % self.o_date.strftime('%d/%m/%Y')
            description = "OFFERTA %s%s" % (
                self.o_number,
                o_date
            )
            if "quotation" in ls.INSTALLED_APPS:
                if self.last_cus_quotation:
                    description = "OFFERTA %s del %s" % (
                        self.last_cus_quotation.number,
                        self.last_cus_quotation.date_q.strftime('%d/%m/%Y')
                    )
                    if self.last_cus_quotation.accepted and self.last_cus_procedure:
                        description = "COMMESSA %s del %s" % (
                            self.last_cus_procedure.number,
                            self.last_cus_procedure.date_p.strftime('%d/%m/%Y')
                        )
            return description
        get_description.short_description = _("Offerta/Commessa")

    if "quotation" in ls.INSTALLED_APPS:
        def get_cus_quotations(self):
            """
            restituisce i preventivi cliente associati al progetto
            """
            return self.quotations.filter(store_cause__category="cus").order_by("-date_q", "-number")

    if "quotation" in ls.INSTALLED_APPS:
        def get_sup_quotations(self):
            """
            restituisce i preventivi fornitore associati al progetto
            """
            return self.quotations.filter(store_cause__category="sup").order_by("-date_q", "-number")
            
    if "quotation" in ls.INSTALLED_APPS:
        def get_last_cus_quotation(self):
            """
            restituisce tra le varie quotation appartenenti al progetto:
            - preventivo accettato cliente
            - ultimo preventivo cliente
            """
            quotations = self.quotations.filter(store_cause__category="cus").order_by("-number")
            if quotations:
                acc_quotations = quotations.filter(accepted="True")
                if acc_quotations:
                    return acc_quotations[0]
                return quotations[0]
            return None
        last_cus_quotation = property(get_last_cus_quotation)

    if "quotation" in ls.INSTALLED_APPS:
        def get_totals_cus_quotations(self):
            """
            restituisce i totali dei preventivi cliente
            """
            total_income = 0
            total_fee = 0
            total_discount = 0
            total_vat = 0
            total_no_vat = 0
            total = 0
            for quotation in self.get_cus_quotations():
                total_income += quotation.totals['income']
                total_fee += quotation.totals['fee']
                total_discount += quotation.totals['discount']
                total_vat += quotation.totals['vat']
                total_no_vat += quotation.totals['total_no_vat']
                total += quotation.totals['total']
                
            totals = {
                'total_income':     total_income,
                'total_fee':        total_fee,
                'total_discount':   total_discount,
                'total_vat':        total_vat,
                'total_no_vat':     total_no_vat,
                'total':            total
            }
            return totals

    if "procedure" in ls.INSTALLED_APPS:
        def get_cus_procedures(self):
            """
            restituisce gli ordini cliente associati al progetto
            """
            return self.procedures.filter(store_cause__category="cus").order_by("-date_p", "-number")

    if "procedure" in ls.INSTALLED_APPS:
        def get_sup_procedures(self):
            """
            restituisce gli ordini fornitore associati al progetto
            """
            return self.procedures.filter(store_cause__category="sup").order_by("-date_p", "-number")

    if "procedure" in ls.INSTALLED_APPS:
        def get_totals_sup_procedures(self):
            """
            restituisce i totali degli ordini fornitore
            """
            total_income = 0
            total_fee = 0
            total_discount = 0
            total_vat = 0
            total_no_vat = 0
            total = 0
            for procedure in self.get_sup_procedures():
                total_income += procedure.totals['income']
                total_fee += procedure.totals['fee']
                total_discount += procedure.totals['discount']
                total_vat += procedure.totals['vat']
                total_no_vat += procedure.totals['total_no_vat']
                total += procedure.totals['total']
                
            totals = {
                'total_income':     total_income,
                'total_fee':        total_fee,
                'total_discount':   total_discount,
                'total_vat':        total_vat,
                'total_no_vat':     total_no_vat,
                'total':            total
            }
            return totals
 
    if "procedure" in ls.INSTALLED_APPS:
        def get_last_cus_procedure(self):
            """
            restituisce tra le varie procedure (ordini) appartenenti al progetto:
            - ultimo ordine cliente
            """
            procedures = self.procedures.filter(store_cause__category="cus").order_by("-number")
            if procedures:
                return procedures[0]
            return None
        last_cus_procedure = property(get_last_cus_procedure)

    if "invoice" in ls.INSTALLED_APPS:
        def get_cus_total(self):
            rows = self.invoicerows.filter(store_cause__category="cus")
            total_income = 0
            total_fee = 0
            total_discount = 0
            for row in rows:
                total_income += row.total_income
                total_fee += row.total_fee
                total_discount += row.total_discount
            totals = {'total_income': total_income, 'total_fee': total_fee, 'total_discount': total_discount, 'total': total_income+total_fee-total_discount}
            return totals

    if "invoice" in ls.INSTALLED_APPS:
        def get_sup_total(self):
            rows = self.invoicerows.filter(store_cause__category="sup")
            total_income = 0
            total_fee = 0
            total_discount = 0
            for row in rows:
                total_income += row.total_income
                total_fee += row.total_fee
                total_discount += row.total_discount
            totals = {'total_income': total_income, 'total_fee': total_fee, 'total_discount': total_discount, 'total': total_income+total_fee-total_discount}
            return totals
