import re

from django.forms import ValidationError
from django.forms.fields import RegexField
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import smart_unicode
from django.contrib.localflavor.it.util import ssn_check_digit, vat_number_check_digit

class ITFiscalCodeField(RegexField):
    """
    A form field that validates either Italian Social Security numbers (codice fiscale)
    or Italian VAT numbers (partita IVA).
    For reference see http://www.agenziaentrate.it/ and search for
    'Informazioni sulla codificazione delle persone fisiche'.
    """
    default_error_messages = {
        'invalid': _(u'Enter a valid Social Security number or a valid VAT number.'),
    }

    def __init__(self, *args, **kwargs):
        super(ITFiscalCodeField, self).__init__(r'(^\w{3}\s*\w{3}\s*\w{5}\s*\w{5}$|^\d{0,11}$)', max_length=None, min_length=None, *args, **kwargs)

    def clean(self, value):
        value = super(ITFiscalCodeField, self).clean(value)
        if value == u'':
            return value
        if re.match('^\d+$', value):
            # check if it is a VAT number
            try:
                vat_number = int(value)
            except ValueError:
                raise ValidationError(self.error_messages['invalid'])
            vat_number = str(vat_number).zfill(11)
            check_digit = vat_number_check_digit(vat_number[0:10])
            if not vat_number[10] == check_digit:
                raise ValidationError(self.error_messages['invalid'])
            return smart_unicode(vat_number)
        else:
            # check if it is a SSN number
            value = re.sub('\s', u'', value).upper()
            try:
                check_digit = ssn_check_digit(value)
            except ValueError:
                raise ValidationError(self.error_messages['invalid'])
            if not value[15] == check_digit:
                raise ValidationError(self.error_messages['invalid'])
            return value

