from django.views.generic import list_detail
from django.contrib.auth.decorators import login_required
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.template import loader, RequestContext
from django.http import Http404, HttpResponse
from django.core.xheaders import populate_xheaders
from django.core.paginator import Paginator, InvalidPage
from django.core.exceptions import ObjectDoesNotExist

from jumbo.apps.tables import get_query_set
from jumbo.http import HttpForbidden
from jumbo.shortcuts import render_to_response, render_to_string

@login_required
def object_detail(request, model, object_id, template_name=None):
    if not template_name:
        template_name = "%s/%s_detail.html" % (model._meta.app_label, model._meta.object_name.lower())
    try:
        obj = model.objects.get(pk=object_id)
    except model.DoesNotExists:
        return render_to_response('jumbo/generic_error.html', error=_("The object does not exists."))

    return render_to_response(template_name, obj=obj)

@login_required
def object_list(request, data_list, search_form=None,
        paginate_by=20, page=None,
        allow_empty=True, template_name=None,
        template_loader=loader, extra_context=None,
        context_processors=None, template_object_name='object',
        mimetype=None):

    dl = data_list()

    if hasattr(dl, 'check_permissions'):
        if not dl.check_permissions(request):
            return HttpForbidden()

    sf = None
    if search_form:
        if request.GET:
            sf = search_form(request.GET)
        else:
            sf = search_form()

    qs = get_query_set(data_list, sf)

    if extra_context:
        extra_context.update({'search_form':search_form, 'data_list':data_list})
    else:
        extra_context = {'search_form':search_form, 'data_list':data_list}

    return list_detail.object_list(
        request,
        queryset=qs,
        paginate_by=paginate_by,
        page=page,
        allow_empty=allow_empty,
        template_name=template_name,
        template_loader=template_loader,
        extra_context=extra_context,
        context_processors=context_processors,
        template_object_name=template_object_name,
        mimetype=mimetype,
    )


#def object_detail(request, queryset,
        #object_id=None, slug=None, slug_field='slug',
        #template_name=None, template_name_field=None,
        #template_loader=loader, extra_context=None,
        #context_processors=None, template_object_name='object',
        #mimetype=None):

    #if callable(queryset):
        #queryset = queryset()

    #return list_detail.object_detail(
        #request,
        #queryset=queryset,
        #object_id=object_id,
        #slug=slug,
        #slug_field=slug_field,
        #template_name=template_name,
        #template_name_field=template_name_field,
        #template_loader=template_loader,
        #extra_context=extra_context,
        #context_processors=context_processors,
        #template_object_name=template_object_name,
        #mimetype=mimetype,
    #)

@login_required
def sortable_object_list(request, queryset, paginate_by=None, page=None,
        allow_empty=True, template_name=None, template_loader=loader,
        extra_context=None, context_processors=None, template_object_name='object',
        mimetype=None):

    if extra_context is None: extra_context = {}
    queryset = queryset._clone()

    next_mod = 'asc'
    if request.GET.has_key('order_by'):
        o = request.GET['order_by']
        queryset = queryset.order_by('-' + o)
        if request.GET['mod'] == 'asc':
            next_mod = 'desc'
            queryset = queryset.order_by(o)

    if paginate_by:
        paginator = Paginator(queryset, paginate_by, allow_empty_first_page=allow_empty)
        if not page:
            page = request.GET.get('page', 1)
        try:
            page_number = int(page)
        except ValueError:
            if page == 'last':
                page_number = paginator.num_pages
            else:
                # Page is not 'last', nor can it be converted to an int.
                raise Http404
        try:
            page_obj = paginator.page(page_number)
        except InvalidPage:
            raise Http404
        c = RequestContext(request, {
            '%s_list' % template_object_name: page_obj.object_list,
            'next_mod': next_mod,
            'paginator': paginator,
            'page_obj': page_obj,

            # Legacy template context stuff. New templates should use page_obj
            # to access this instead.
            'is_paginated': page_obj.has_other_pages(),
            'results_per_page': paginator.per_page,
            'has_next': page_obj.has_next(),
            'has_previous': page_obj.has_previous(),
            'page': page_obj.number,
            'next': page_obj.next_page_number(),
            'previous': page_obj.previous_page_number(),
            'first_on_page': page_obj.start_index(),
            'last_on_page': page_obj.end_index(),
            'pages': paginator.num_pages,
            'hits': paginator.count,
            'page_range': paginator.page_range,
        }, context_processors)
    else:
        c = RequestContext(request, {
            '%s_list' % template_object_name: queryset,
            'next_mod': next_mod,
            'paginator': None,
            'page_obj': None,
            'is_paginated': False,
        }, context_processors)
        if not allow_empty and len(queryset) == 0:
            raise Http404
    for key, value in extra_context.items():
        if callable(value):
            c[key] = value()
        else:
            c[key] = value
    if not template_name:
        model = queryset.model
        template_name = "%s/%s_list.html" % (model._meta.app_label, model._meta.object_name.lower())
    t = template_loader.get_template(template_name)
    return HttpResponse(t.render(c), mimetype=mimetype)
