# taken and modified from django.views.static.py

"""
Views and functions for serving static files. These are only to be used
during development, and SHOULD NOT be used in a production setting.
"""

import mimetypes
import os
import posixpath
import stat
import urllib

from django.http import Http404, HttpResponse, HttpResponseRedirect, HttpResponseNotModified
from django.utils.http import http_date
from django.utils.translation import ugettext as _
from django.views.static import was_modified_since

def serve(request, path, document_roots=None):
    """
    Serve static files below a given point in the directory structure.

    To use, put a URL pattern such as::

        (r'^%s/(?P<path>.*)' % settings.MEDIA_URL[1:-1], 'jumbo.views.static.serve', {'document_roots': (settings.MEDIA_ROOT, settings.JUMBO_MEDIA_ROOT,),}),

    in your URLconf. You must provide the ``document_roots`` param, a tuple of directory paths.
    """

    # Clean up given path to only allow serving files below document_root.
    path = posixpath.normpath(urllib.unquote(path))
    path = path.lstrip('/')
    newpath = ''

    for part in path.split('/'):
        if not part:
            # Strip empty path components.
            continue
        drive, part = os.path.splitdrive(part)
        head, part = os.path.split(part)
        if part in (os.curdir, os.pardir):
            # Strip '.' and '..' in path.
            continue
        newpath = os.path.join(newpath, part).replace('\\', '/')

    if newpath and path != newpath:
        return HttpResponseRedirect(newpath)

    for document_root in document_roots:
        fullpath = os.path.join(document_root, newpath)
        if os.path.isdir(fullpath) or not os.path.exists(fullpath):
            continue
        # Respect the If-Modified-Since header.
        statobj = os.stat(fullpath)
        if not was_modified_since(request.META.get('HTTP_IF_MODIFIED_SINCE'),
                                  statobj[stat.ST_MTIME], statobj[stat.ST_SIZE]):
            return HttpResponseNotModified()
        mimetype = mimetypes.guess_type(fullpath)[0] or 'application/octet-stream'
        contents = open(fullpath, 'rb').read()
        response = HttpResponse(contents, mimetype=mimetype)
        response["Last-Modified"] = http_date(statobj[stat.ST_MTIME])
        response["Content-Length"] = len(contents)
        return response

    raise Http404, _('"%s" does not exist') % path

